#!/usr/bin/python -tt
from __future__ import with_statement
import optparse
import sys
import os
import subprocess

#output "constants"
LOG_FATAL  = 0
LOG_ERROR  = 1
LOG_WARN   = 2
LOG_ALWAYS = 2.5
LOG_NORMAL = 3
LOG_INFO   = 4
LOG_DEBUG  = 5

msgPrefixes = [
    "FATAL: ",
    "ERROR: ",
    "WARNING: ",
    "",
    "INFO: ",
    "DEBUG: "
]

#local variables
null = open(os.devnull, 'r+')
parser = optparse.OptionParser()
options = None

def dieCleanly(level = None):
    """Exit the application with proper cleanup."""

    #TODO: perform application cleanup

    if level == None:
        level = LOG_ERROR

    #exit with appropriate value
    if level == LOG_FATAL:
        sys.exit(1)
    sys.exit(0)


def message(level, msg):
    global options
    """Print a message to a certain debug level"""
    retval = None

    if level <= options.logLevel or level == LOG_ALWAYS:
        out = sys.stdout

        # if logfile is open print to it instead
        if options.logFile == "-":
            out = sys.log
        elif level <= LOG_WARN:
            out = sys.stderr

        retval = msgPrefixes[int(level + 0.5)] + msg
        out.write(retval)
        retval = len(retval)

    if level <= LOG_FATAL:
        dieCleanly(level)

    return retval

def printUsage(msg = None):
    global parser
    if msg != None:
        message(LOG_FATAL, msg + parser.get_usage())
    message(LOG_ALWAYS, usage)
    dieCleanly(LOG_ALWAYS)

def parseOptions():
    global parser, options
    parser.add_option('-l', '--log-file',
                      metavar = 'FILE', dest = 'logFile',
                      help = 'Specify a log to receive all messages.')
    parser.add_option('--loops',
                      metavar = 'NUM', type = 'int', default = '1',
                      help = 'Run the command this many times.')
    parser.add_option('-q', '--quiet',
                      action = 'count',
                      help = 'Decrease verbosity.')
    parser.add_option('-v', '--verbose',
                      action = 'count',
                      help = 'Increase verbosity.')
    (options, leftover) = parser.parse_args()
    options.logLevel = LOG_NORMAL;
    if options.verbose:
        options.logLevel += options.verbose
    if options.quiet:
        options.logLevel -= options.quiet
        if options.logLevel <= LOG_FATAL:
            options.logLevel = LOG_FATAL
    if options.logFile == '-':
        options.logFile = None
    options.args = leftover

parseOptions()
# open the log file if specified
if options.logFile != None:
    sys.log = open(options.logFile, "a", 1)
    options.logFile = "-"

cmd = ['igclient']
cmd.extend(map(lambda a : a.replace('=', '-'), options.args))

expected = None

count = 0
while count < options.loops:
    igclient = subprocess.Popen(cmd,
                                stdin = subprocess.PIPE,
                                stdout = subprocess.PIPE,
                                stderr = subprocess.STDOUT)
    igclient.stdin.close()
    igclient.wait
    result = igclient.stdout.read()
    if expected is None:
        expected = result
        message(LOG_NORMAL, 'Expecting all runs to return:\n %s\n' % expected)
    elif expected != result:
        message(LOG_ERROR, 'Unexpected result: %s\n' % result)
    count += 1
    sys.stdout.write('%d calls complete\r' % count)
    sys.stdout.flush()
message(LOG_NORMAL, '\nAll done\n')
